@@ -9,7 +9,7 @@ use std::iter::Peekable;
 use std::str::CharIndices;
 use unicode_xid::UnicodeXID;
 
-use crate::pt::{Comment, Loc};
+use crate::pt::{CodeLocation, Comment, Loc};
 
 pub type Spanned<Token, Loc, Error> = Result<(Loc, Token, Loc), Error>;
 
@@ -68,6 +68,7 @@ pub enum Token<'input> {
     Emit,
     Return,
     Returns,
+    Revert,
 
     Uint(u16),
     Int(u16),
@@ -174,12 +175,14 @@ pub enum Token<'input> {
     Modifier,
     Immutable,
     Unchecked,
+
     Assembly,
     Let,
     Leave,
     Switch,
     Case,
     Default,
+    YulArrow,
 }
 
 impl<'input> fmt::Display for Token<'input> {
@@ -283,6 +286,7 @@ impl<'input> fmt::Display for Token<'input> {
             Token::Emit => write!(f, "emit"),
             Token::Return => write!(f, "return"),
             Token::Returns => write!(f, "returns"),
+            Token::Revert => write!(f, "revert"),
             Token::True => write!(f, "true"),
             Token::False => write!(f, "false"),
             Token::Else => write!(f, "else"),
@@ -323,6 +327,7 @@ impl<'input> fmt::Display for Token<'input> {
             Token::Switch => write!(f, "switch"),
             Token::Case => write!(f, "case"),
             Token::Default => write!(f, "default"),
+            Token::YulArrow => write!(f, "->"),
         }
     }
 }
@@ -374,19 +379,19 @@ impl fmt::Display for LexicalError {
     }
 }
 
-impl LexicalError {
-    pub fn loc(&self) -> &Loc {
+impl CodeLocation for LexicalError {
+    fn loc(&self) -> Loc {
         match self {
-            LexicalError::EndOfFileInComment(loc, ..) => loc,
-            LexicalError::EndOfFileInString(loc, ..) => loc,
-            LexicalError::EndofFileInHex(loc, ..) => loc,
-            LexicalError::MissingNumber(loc, ..) => loc,
-            LexicalError::InvalidCharacterInHexLiteral(loc, _) => loc,
-            LexicalError::UnrecognisedToken(loc, ..) => loc,
-            LexicalError::ExpectedFrom(loc, ..) => loc,
-            LexicalError::MissingExponent(loc, ..) => loc,
-            LexicalError::DoublePoints(loc, ..) => loc,
-            LexicalError::UnrecognisedDecimal(loc, ..) => loc,
+            LexicalError::EndOfFileInComment(loc, ..)
+            | LexicalError::EndOfFileInString(loc, ..)
+            | LexicalError::EndofFileInHex(loc, ..)
+            | LexicalError::MissingNumber(loc, ..)
+            | LexicalError::InvalidCharacterInHexLiteral(loc, _)
+            | LexicalError::UnrecognisedToken(loc, ..)
+            | LexicalError::ExpectedFrom(loc, ..)
+            | LexicalError::MissingExponent(loc, ..)
+            | LexicalError::DoublePoints(loc, ..)
+            | LexicalError::UnrecognisedDecimal(loc, ..) => *loc,
         }
     }
 }
@@ -498,6 +503,7 @@ static KEYWORDS: phf::Map<&'static str, Token> = phf_map! {
     "pure" => Token::Pure,
     "returns" => Token::Returns,
     "return" => Token::Return,
+    "revert" => Token::Revert,
     "storage" => Token::Storage,
     "string" => Token::String,
     "struct" => Token::Struct,
@@ -591,15 +597,15 @@ impl<'input> Lexer<'input> {
 
                 let mut end = match self.chars.next() {
                     Some((end, ch)) if ch.is_ascii_hexdigit() => end,
-                    Some((_, _)) => {
-                        return Err(LexicalError::MissingNumber(Loc(
+                    Some((..)) => {
+                        return Err(LexicalError::MissingNumber(Loc::File(
                             self.file_no,
                             start,
                             start + 1,
                         )));
                     }
                     None => {
-                        return Err(LexicalError::EndofFileInHex(Loc(
+                        return Err(LexicalError::EndofFileInHex(Loc::File(
                             self.file_no,
                             start,
                             self.input.len(),
@@ -643,7 +649,7 @@ impl<'input> Lexer<'input> {
 
         if let Some((i, '.')) = self.chars.peek() {
             if is_rational {
-                return Err(LexicalError::DoublePoints(Loc(
+                return Err(LexicalError::DoublePoints(Loc::File(
                     self.file_no,
                     start,
                     self.input.len(),
@@ -656,7 +662,7 @@ impl<'input> Lexer<'input> {
             self.chars.next();
             while let Some((i, ch)) = self.chars.peek() {
                 if *ch == '.' {
-                    return Err(LexicalError::DoublePoints(Loc(
+                    return Err(LexicalError::DoublePoints(Loc::File(
                         self.file_no,
                         start,
                         self.input.len(),
@@ -671,7 +677,7 @@ impl<'input> Lexer<'input> {
                 self.chars.next();
             }
             if !has_number {
-                return Err(LexicalError::UnrecognisedDecimal(Loc(
+                return Err(LexicalError::UnrecognisedDecimal(Loc::File(
                     self.file_no,
                     start,
                     self.input.len(),
@@ -694,7 +700,7 @@ impl<'input> Lexer<'input> {
             }
 
             if exp_start > end {
-                return Err(LexicalError::MissingExponent(Loc(
+                return Err(LexicalError::MissingExponent(Loc::File(
                     self.file_no,
                     start,
                     self.input.len(),
@@ -707,7 +713,7 @@ impl<'input> Lexer<'input> {
             let mantissa = &self.input[rational_start..=rational_end];
 
             if mantissa.is_empty() {
-                return Err(LexicalError::UnrecognisedDecimal(Loc(
+                return Err(LexicalError::UnrecognisedDecimal(Loc::File(
                     self.file_no,
                     start,
                     self.input.len(),
@@ -749,7 +755,7 @@ impl<'input> Lexer<'input> {
                     last_was_escape = false;
                 }
             } else {
-                return Err(LexicalError::EndOfFileInString(Loc(
+                return Err(LexicalError::EndOfFileInString(Loc::File(
                     self.file_no,
                     token_start,
                     self.input.len(),
@@ -824,14 +830,14 @@ impl<'input> Lexer<'input> {
 
                                         return Some(Err(
                                             LexicalError::InvalidCharacterInHexLiteral(
-                                                Loc(self.file_no, i, i + 1),
+                                                Loc::File(self.file_no, i, i + 1),
                                                 ch,
                                             ),
                                         ));
                                     }
                                 }
 
-                                return Some(Err(LexicalError::EndOfFileInString(Loc(
+                                return Some(Err(LexicalError::EndOfFileInString(Loc::File(
                                     self.file_no,
                                     start,
                                     self.input.len(),
@@ -858,7 +864,7 @@ impl<'input> Lexer<'input> {
                                     }
                                 }
 
-                                return Some(Err(LexicalError::EndOfFileInString(Loc(
+                                return Some(Err(LexicalError::EndOfFileInString(Loc::File(
                                     self.file_no,
                                     start,
                                     self.input.len(),
@@ -905,10 +911,17 @@ impl<'input> Lexer<'input> {
                             let mut last = start + 3;
 
                             if !newline {
-                                for (i, ch) in &mut self.chars {
-                                    last = i;
-                                    if ch == '\n' || ch == '\r' {
-                                        break;
+                                loop {
+                                    match self.chars.next() {
+                                        None => {
+                                            last = self.input.len();
+                                            break;
+                                        }
+                                        Some((offset, '\n' | '\r')) => {
+                                            last = offset;
+                                            break;
+                                        }
+                                        Some(_) => (),
                                     }
                                 }
                             }
@@ -926,7 +939,7 @@ impl<'input> Lexer<'input> {
                                 }
                             } else {
                                 self.comments.push(Comment::Line(
-                                    Loc(self.file_no, start, last),
+                                    Loc::File(self.file_no, start, last),
                                     self.input[start..last].to_owned(),
                                 ));
                             }
@@ -954,7 +967,7 @@ impl<'input> Lexer<'input> {
                                     seen_star = ch == '*';
                                     last = i;
                                 } else {
-                                    return Some(Err(LexicalError::EndOfFileInComment(Loc(
+                                    return Some(Err(LexicalError::EndOfFileInComment(Loc::File(
                                         self.file_no,
                                         start,
                                         self.input.len(),
@@ -975,7 +988,7 @@ impl<'input> Lexer<'input> {
                                 }
                             } else {
                                 self.comments.push(Comment::Block(
-                                    Loc(self.file_no, start, last + 2),
+                                    Loc::File(self.file_no, start, last + 2),
                                     self.input[start..last + 2].to_owned(),
                                 ));
                             }
@@ -1074,6 +1087,10 @@ impl<'input> Lexer<'input> {
                             self.chars.next();
                             Some(Ok((i, Token::Decrement, i + 2)))
                         }
+                        Some((_, '>')) => {
+                            self.chars.next();
+                            Some(Ok((i, Token::YulArrow, i + 2)))
+                        }
                         _ => Some(Ok((i, Token::Subtract, i + 1))),
                     };
                 }
@@ -1173,7 +1190,7 @@ impl<'input> Lexer<'input> {
                     }
 
                     return Some(Err(LexicalError::UnrecognisedToken(
-                        Loc(self.file_no, start, end),
+                        Loc::File(self.file_no, start, end),
                         self.input[start..end].to_owned(),
                     )));
                 }
@@ -1425,7 +1442,7 @@ fn lexertest() {
     assert_eq!(
         tokens,
         vec!(Err(LexicalError::InvalidCharacterInHexLiteral(
-            Loc(0, 10, 11),
+            Loc::File(0, 10, 11),
             'g'
         )))
     );
@@ -1436,7 +1453,7 @@ fn lexertest() {
     assert_eq!(
         tokens,
         vec!(Err(LexicalError::UnrecognisedToken(
-            Loc(0, 1, 4),
+            Loc::File(0, 1, 4),
             "€".to_owned()
         )))
     );
@@ -1447,7 +1464,7 @@ fn lexertest() {
     assert_eq!(
         tokens,
         vec!(Err(LexicalError::UnrecognisedToken(
-            Loc(0, 0, 3),
+            Loc::File(0, 0, 3),
             "€".to_owned()
         )))
     );
@@ -1510,7 +1527,7 @@ fn lexertest() {
     let tokens = Lexer::new("/**", 0, &mut comments).next();
     assert_eq!(
         tokens,
-        Some(Err(LexicalError::EndOfFileInComment(Loc(0, 0, 3))))
+        Some(Err(LexicalError::EndOfFileInComment(Loc::File(0, 0, 3))))
     );
 
     let tokens = Lexer::new("//////////////", 0, &mut comments).next();
@@ -1575,7 +1592,7 @@ fn lexertest() {
         tokens,
         vec!(
             Ok((1, Token::Subtract, 2)),
-            Err(LexicalError::MissingExponent(Loc(0, 2, 4)))
+            Err(LexicalError::MissingExponent(Loc::File(0, 2, 4)))
         )
     );
 
@@ -1585,7 +1602,7 @@ fn lexertest() {
     assert_eq!(
         tokens,
         vec!(
-            Err(LexicalError::MissingExponent(Loc(0, 0, 3))),
+            Err(LexicalError::MissingExponent(Loc::File(0, 0, 3))),
             Ok((2, Token::Identifier("a"), 3))
         )
     );
