@@ -7,17 +7,78 @@ use crate::lexer::CommentType;
 
 #[derive(Debug, PartialEq, PartialOrd, Ord, Eq, Hash, Clone, Copy)]
 /// file no, start offset, end offset (in bytes)
-pub struct Loc(pub usize, pub usize, pub usize);
+pub enum Loc {
+    Builtin,
+    CommandLine,
+    Implicit,
+    Codegen,
+    File(usize, usize, usize),
+}
+
+/// Structs can implement this trait to easily return their loc
+pub trait CodeLocation {
+    fn loc(&self) -> Loc;
+}
+
+/// Structs should implement this trait to return an optional location
+pub trait OptionalCodeLocation {
+    fn loc(&self) -> Option<Loc>;
+}
 
 impl Loc {
     #[must_use]
-    pub fn begin(&self) -> Self {
-        Loc(self.0, self.1, self.1)
+    pub fn begin_range(&self) -> Self {
+        match self {
+            Loc::File(file_no, start, _) => Loc::File(*file_no, *start, *start),
+            loc => *loc,
+        }
     }
 
     #[must_use]
-    pub fn end(&self) -> Self {
-        Loc(self.0, self.2, self.2)
+    pub fn end_range(&self) -> Self {
+        match self {
+            Loc::File(file_no, _, end) => Loc::File(*file_no, *end, *end),
+            loc => *loc,
+        }
+    }
+
+    pub fn file_no(&self) -> usize {
+        match self {
+            Loc::File(file_no, _, _) => *file_no,
+            _ => unreachable!(),
+        }
+    }
+
+    pub fn start(&self) -> usize {
+        match self {
+            Loc::File(_, start, _) => *start,
+            _ => unreachable!(),
+        }
+    }
+
+    pub fn end(&self) -> usize {
+        match self {
+            Loc::File(_, _, end) => *end,
+            _ => unreachable!(),
+        }
+    }
+
+    pub fn use_end_from(&mut self, other: &Loc) {
+        match (self, other) {
+            (Loc::File(_, _, end), Loc::File(_, _, other_end)) => {
+                *end = *other_end;
+            }
+            _ => unreachable!(),
+        }
+    }
+
+    pub fn use_start_from(&mut self, other: &Loc) {
+        match (self, other) {
+            (Loc::File(_, start, _), Loc::File(_, other_start, _)) => {
+                *start = *other_start;
+            }
+            _ => unreachable!(),
+        }
     }
 }
 
@@ -107,12 +168,12 @@ pub enum StorageLocation {
     Calldata(Loc),
 }
 
-impl StorageLocation {
-    pub fn loc(&self) -> &Loc {
+impl CodeLocation for StorageLocation {
+    fn loc(&self) -> Loc {
         match self {
-            StorageLocation::Memory(l) => l,
-            StorageLocation::Storage(l) => l,
-            StorageLocation::Calldata(l) => l,
+            StorageLocation::Memory(l)
+            | StorageLocation::Storage(l)
+            | StorageLocation::Calldata(l) => *l,
         }
     }
 }
@@ -358,8 +419,8 @@ pub enum Expression {
     This(Loc),
 }
 
-impl Expression {
-    pub fn loc(&self) -> Loc {
+impl CodeLocation for Expression {
+    fn loc(&self) -> Loc {
         match self {
             Expression::PostIncrement(loc, _)
             | Expression::PostDecrement(loc, _)
@@ -451,8 +512,8 @@ impl fmt::Display for Mutability {
     }
 }
 
-impl Mutability {
-    pub fn loc(&self) -> Loc {
+impl CodeLocation for Mutability {
+    fn loc(&self) -> Loc {
         match self {
             Mutability::Pure(loc)
             | Mutability::Constant(loc)
@@ -481,8 +542,8 @@ impl fmt::Display for Visibility {
     }
 }
 
-impl Visibility {
-    pub fn loc(&self) -> Option<Loc> {
+impl OptionalCodeLocation for Visibility {
+    fn loc(&self) -> Option<Loc> {
         match self {
             Visibility::Public(loc)
             | Visibility::External(loc)
@@ -546,7 +607,8 @@ pub enum Statement {
     },
     Assembly {
         loc: Loc,
-        assembly: AssemblyBlock,
+        dialect: Option<StringLiteral>,
+        block: YulBlock,
     },
     Args(Loc, Vec<NamedArgument>),
     If(Loc, Expression, Box<Statement>, Option<Box<Statement>>),
@@ -564,6 +626,7 @@ pub enum Statement {
     Continue(Loc),
     Break(Loc),
     Return(Loc, Option<Expression>),
+    Revert(Loc, Option<Identifier>, Vec<Expression>),
     Emit(Loc, Expression),
     Try(
         Loc,
@@ -581,53 +644,93 @@ pub enum CatchClause {
 }
 
 #[derive(Debug, PartialEq, Clone)]
-pub enum AssemblyStatement {
-    Assign(Loc, AssemblyExpression, AssemblyExpression),
-    LetAssign(Loc, AssemblyExpression, AssemblyExpression),
-    Expression(AssemblyExpression),
-    If(Loc, AssemblyExpression, Box<AssemblyBlock>),
-    For(
-        Loc,
-        AssemblyBlock,
-        AssemblyExpression,
-        AssemblyBlock,
-        Box<AssemblyBlock>,
-    ),
-    Switch(
-        Loc,
-        AssemblyExpression,
-        Vec<AssemblySwitch>,
-        Option<AssemblySwitch>,
-    ),
+pub enum YulStatement {
+    Assign(Loc, Vec<YulExpression>, YulExpression),
+    VariableDeclaration(Loc, Vec<YulTypedIdentifier>, Option<YulExpression>),
+    If(Loc, YulExpression, YulBlock),
+    For(YulFor),
+    Switch(YulSwitch),
     Leave(Loc),
     Break(Loc),
     Continue(Loc),
+    Block(YulBlock),
+    FunctionDefinition(Box<YulFunctionDefinition>),
+    FunctionCall(Box<YulFunctionCall>),
+}
+#[derive(PartialEq, Clone, Debug)]
+pub struct YulSwitch {
+    pub loc: Loc,
+    pub condition: YulExpression,
+    pub cases: Vec<YulSwitchOptions>,
+    pub default: Option<YulSwitchOptions>,
+}
+
+#[derive(PartialEq, Clone, Debug)]
+pub struct YulFor {
+    pub loc: Loc,
+    pub init_block: YulBlock,
+    pub condition: YulExpression,
+    pub post_block: YulBlock,
+    pub execution_block: YulBlock,
 }
 
 #[derive(Debug, PartialEq, Clone)]
-pub enum AssemblyExpression {
-    BoolLiteral(Loc, bool),
-    NumberLiteral(Loc, BigInt),
-    HexNumberLiteral(Loc, String),
-    StringLiteral(StringLiteral),
+pub struct YulBlock {
+    pub loc: Loc,
+    pub statements: Vec<YulStatement>,
+}
+
+#[derive(Debug, PartialEq, Clone)]
+pub enum YulExpression {
+    BoolLiteral(Loc, bool, Option<Identifier>),
+    NumberLiteral(Loc, BigInt, Option<Identifier>),
+    HexNumberLiteral(Loc, String, Option<Identifier>),
+    HexStringLiteral(HexLiteral, Option<Identifier>),
+    StringLiteral(StringLiteral, Option<Identifier>),
     Variable(Identifier),
-    Assign(Loc, Box<AssemblyExpression>, Box<AssemblyExpression>),
-    LetAssign(Loc, Box<AssemblyExpression>, Box<AssemblyExpression>),
-    Function(Loc, Box<AssemblyExpression>, Vec<AssemblyExpression>),
-    Member(Loc, Box<AssemblyExpression>, Identifier),
-    Subscript(Loc, Box<AssemblyExpression>, Box<AssemblyExpression>),
+    FunctionCall(Box<YulFunctionCall>),
+    Member(Loc, Box<YulExpression>, Identifier),
 }
 
-pub type AssemblyBlock = Vec<AssemblyStatement>;
-
 #[derive(Debug, PartialEq, Clone)]
-pub enum AssemblySwitch {
-    Case(AssemblyExpression, Box<AssemblyBlock>),
-    Default(Box<AssemblyBlock>),
+pub struct YulTypedIdentifier {
+    pub loc: Loc,
+    pub id: Identifier,
+    pub ty: Option<Identifier>,
 }
 
-impl Statement {
-    pub fn loc(&self) -> Loc {
+#[derive(Debug, PartialEq, Clone)]
+pub struct YulFunctionDefinition {
+    pub loc: Loc,
+    pub id: Identifier,
+    pub params: Vec<YulTypedIdentifier>,
+    pub returns: Vec<YulTypedIdentifier>,
+    pub body: YulBlock,
+}
+
+#[derive(Debug, PartialEq, Clone)]
+pub struct YulFunctionCall {
+    pub loc: Loc,
+    pub id: Identifier,
+    pub arguments: Vec<YulExpression>,
+}
+
+#[derive(Debug, PartialEq, Clone)]
+pub enum YulSwitchOptions {
+    Case(Loc, YulExpression, YulBlock),
+    Default(Loc, YulBlock),
+}
+
+impl CodeLocation for YulSwitchOptions {
+    fn loc(&self) -> Loc {
+        match self {
+            YulSwitchOptions::Case(loc, ..) | YulSwitchOptions::Default(loc, ..) => *loc,
+        }
+    }
+}
+
+impl CodeLocation for Statement {
+    fn loc(&self) -> Loc {
         match self {
             Statement::Block { loc, .. }
             | Statement::Assembly { loc, .. }
@@ -641,9 +744,32 @@ impl Statement {
             | Statement::Continue(loc)
             | Statement::Break(loc)
             | Statement::Return(loc, ..)
+            | Statement::Revert(loc, ..)
             | Statement::Emit(loc, ..)
             | Statement::Try(loc, ..)
             | Statement::DocComment(loc, ..) => *loc,
         }
     }
 }
+
+impl YulStatement {
+    pub fn loc(&self) -> Loc {
+        match self {
+            YulStatement::Assign(loc, ..)
+            | YulStatement::VariableDeclaration(loc, ..)
+            | YulStatement::If(loc, ..)
+            | YulStatement::Leave(loc, ..)
+            | YulStatement::Break(loc, ..)
+            | YulStatement::Continue(loc, ..) => *loc,
+
+            YulStatement::Block(block) => block.loc,
+
+            YulStatement::FunctionDefinition(func_def) => func_def.loc,
+
+            YulStatement::FunctionCall(func_call) => func_call.loc,
+
+            YulStatement::For(for_struct) => for_struct.loc,
+            YulStatement::Switch(switch_struct) => switch_struct.loc,
+        }
+    }
+}
